<?php

namespace Drupal\juxtapose\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'Juxtapose Slider' image formatter.
 *
 * @FieldFormatter(
 *   id = "juxtapose_image",
 *   label = @Translation("Juxtapose Before After Slider"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class JuxtaposeImageFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  private EntityTypeManager $entityTypeManager;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * JuxtaposeImageFormatter constructor.
   *
   * @param string $plugin_id
   *   Plugin id.
   * @param string $plugin_definition
   *   Plugin definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   Field definition.
   * @param array $settings
   *   Settings.
   * @param string $label
   *   Label.
   * @param string $view_mode
   *   View mode.
   * @param array $third_party_settings
   *   Third party settings.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   EntityTypeManager.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, EntityTypeManager $entityTypeManager, FileUrlGeneratorInterface $file_url_generator) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->entityTypeManager = $entityTypeManager;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'image_style' => '',
      'starting_position' => '',
      'additional_options' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $settings = $this->getSettings();

    $options = [];
    $options['default'] = $this->t('Original size');

    try {
      $styles = $this->entityTypeManager->getStorage('image_style')->loadMultiple();
      foreach ($styles as $key => $style) {
        $options[$key] = $style->label();
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('juxtapose')->warning($e->getMessage());
    }

    $elements['image_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Image style'),
      '#default_value' => $settings['image_style'],
      '#empty_value' => 'default',
      '#options' => $options,
    ];

    $elements['starting_position'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Starting position'),
      '#description' => $this->t('The starting position is expressed as a percentage, ex: 70.  Defaults to 50 if empty.'),
      '#default_value' => $settings['starting_position'],
      '#placeholder' => 'Ex: 30',
    ];

    $elements['additional_options'] = [
      '#type' => 'textarea',
      '#rows' => 5,
      '#title' => $this->t('Additional options'),
      '#description' => $this->t('Allows to define additional options for the Juxtapose image formatter, ex: show_labels=false to remove labels.'),
      '#default_value' => $settings['additional_options'],
      '#placeholder' => 'Ex: show_labels=false',
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $settings = $this->getSettings();
    $summary = [];

    if ($settings['image_style']) {
      $summary[] = $this->t('Image style: @image_style', ['@image_style' => $settings['image_style']]);
    }

    if ($settings['starting_position']) {
      $summary[] = $this->t('Starting position: @starting_position', ['@starting_position' => $settings['starting_position']]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $settings = $this->getSettings();

    $options = [];
    $options['starting_position'] = $settings['starting_position'] ?? '50%';

    $additional_options = self::parseKeyValueString($settings['additional_options']);
    $options = array_merge($options, $additional_options);

    try {
      /** @var $style \Drupal\image\ImageStyleInterface */
      $style = $this->entityTypeManager->getStorage('image_style')
        ->load($settings['image_style']);
    }
    catch (\Exception $e) {
      \Drupal::logger('juxtapose')->warning($e->getMessage());
    }
    $images = [];

    foreach ($items as $item) {
      $url = isset($style) ? $style->buildUrl($item->entity->getFileUri()) : $this->fileUrlGenerator->generateAbsoluteString($item->entity->getFileUri());
      $alt = $item->get('alt')->getValue();
      $image = [];
      $image['url'] = $url;
      $image['alt'] = $alt;
      $image['label'] = $alt;
      $images[] = $image;
      if (count($images) == 2) {
        break;
      }
    }

    return [
      '#theme' => 'juxtapose_slider_image',
      '#images' => $images,
      '#options' => $options,
      '#attached' => [
        'library' => [
          'juxtapose/juxtapose',
        ],
      ],
    ];
  }

  /**
   * Creates an instance of the plugin.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container to pull out services used in the plugin.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of this plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get('file_url_generator')
    );
  }

  /**
   * Fonction qui analyse une chaîne de caractères multiligne où chaque ligne
   * contient un couple clé-valeur séparé par un signe égal, et qui renvoie un
   * tableau associatif de ces clés et valeurs.
   *
   * @param $string
   *   La chaîne à analyser.
   *
   * @return array
   *   Le tableau associatif contenant les clés et les valeurs.
   */
  static function parseKeyValueString($string): array {
    $result = [];  // Tableau pour stocker les résultats
    $lines = explode("\n", $string);  // Sépare la chaîne en lignes

    foreach ($lines as $line) {
      if (trim($line) != '') {  // Vérifie si la ligne n'est pas vide
        [$key, $value] = explode('=', $line, 2);  // Sépare la ligne en deux au niveau du signe égal
        $key = trim($key);   // Nettoie les espaces blancs autour de la clé
        $value = trim($value);  // Nettoie les espaces blancs autour de la valeur
        if ($key != '') {
          $result[$key] = $value;  // Ajoute le couple clé-valeur au tableau résultat
        }
      }
    }

    return $result;  // Retourne le tableau associatif
  }
}
